@@ -18,7 +18,7 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  *
- * $Id: totemGMPPlugin.h 3408 2006-08-11 13:25:44Z chpe $
+ * $Id: totemGMPPlugin.h 3717 2006-11-15 17:21:16Z chpe $
  */
 
 #ifndef __GMP_PLAYER_H__
@@ -44,7 +44,7 @@ class totemScriptablePlugin : public totemIGMPPlayer,
     totemScriptablePlugin (totemPlugin *aPlugin);
 
     PRBool IsValid () { return mPlugin != nsnull; }
-    void UnsetPlugin () { mPlugin = nsnull; }
+    void SetPlugin (totemPlugin *aPlugin) { mPlugin = aPlugin; }
 
     static char *PluginDescription ();
     static void PluginMimeTypes (const totemPluginMimeEntry **, PRUint32 *);
