@@ -28,24 +28,26 @@
 
 #include "config.h"
 
-#ifndef HAVE_GTK_ONLY
-#include <gnome.h>
-#include "totem-gromit.h"
-#else
 #include <glib/gi18n.h>
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <stdlib.h>
+
+#ifndef HAVE_GTK_ONLY
+#include <gnome.h>
+#include "totem-gromit.h"
 #endif /* !HAVE_GTK_ONLY */
 
 #include <string.h>
 
+#ifdef GDK_WINDOWING_X11
 /* X11 headers */
 #include <gdk/gdkx.h>
 #include <X11/Xlib.h>
 #ifdef HAVE_XFREE
 #include <X11/XF86keysym.h>
 #endif
+#endif
 
 #include "bacon-video-widget.h"
 #include "bacon-video-widget-properties.h"
@@ -55,6 +57,7 @@
 #include "totem-screenshot.h"
 #include "totem-sidebar.h"
 #include "totem-menu.h"
+#include "totem-missing-plugins.h"
 #include "totem-options.h"
 #include "totem-uri.h"
 #include "totem-interface.h"
@@ -64,7 +67,6 @@
 #include "totem.h"
 #include "totem-private.h"
 #include "totem-preferences.h"
-#include "totem-stock-icons.h"
 #include "totem-disc.h"
 
 #include "debug.h"
@@ -80,6 +82,8 @@
 #define ZOOM_UPPER 200
 #define ZOOM_RESET 100
 #define ZOOM_LOWER 10
+#define ZOOM_DISABLE (ZOOM_LOWER - 1)
+#define ZOOM_ENABLE (ZOOM_UPPER + 1)
 
 #define FULLSCREEN_POPUP_TIMEOUT 5 * 1000
 
@@ -105,6 +109,7 @@ static void playlist_changed_cb (GtkWidget *playlist, Totem *totem);
 static void play_pause_set_label (Totem *totem, TotemStates state);
 static gboolean on_video_motion_notify_event (GtkWidget *widget, GdkEventMotion *event, Totem *totem);
 static void popup_timeout_remove (Totem *totem);
+static gint totem_compare_recent_stream_items (GtkRecentInfo *a, GtkRecentInfo *b);
 
 static void
 long_action (void)
@@ -164,10 +169,8 @@ totem_action_save_state (Totem *totem)
 			"show_sidebar", totem_sidebar_is_visible (totem));
 	g_key_file_set_boolean (keyfile, "State",
 			"maximised", totem->maximised);
-	if (totem->maximised == FALSE) {
-		g_key_file_set_integer (keyfile, "State",
-				"sidebar_w", totem->sidebar_w);
-	}
+	g_key_file_set_integer (keyfile, "State",
+			"sidebar_w", totem->sidebar_w);
 
 	page_id = totem_sidebar_get_current_page (totem);
 	g_key_file_set_string (keyfile, "State",
@@ -228,7 +231,6 @@ totem_action_exit (Totem *totem)
 
 	totem_sublang_exit (totem);
 	totem_destroy_file_filters ();
-	totem_named_icons_dispose (totem);
 
 	if (totem->gc)
 		g_object_unref (G_OBJECT (totem->gc));
@@ -266,6 +268,7 @@ play_pause_set_label (Totem *totem, TotemStates state)
 {
 	GtkAction *action;
 	const char *id, *tip;
+	GSList *l, *proxies;
 
 	if (state == totem->state)
 		return;
@@ -301,6 +304,12 @@ play_pause_set_label (Totem *totem, TotemStates state)
 			"tooltip", _(tip),
 			"stock-id", id, NULL);
 
+	proxies = gtk_action_get_proxies (action);
+	for (l = proxies; l != NULL; l = l->next) {
+		atk_object_set_name (gtk_widget_get_accessible (l->data),
+				_(tip));
+	}
+
 	totem->state = state;
 }
 
@@ -332,6 +341,12 @@ totem_action_eject (Totem *totem)
 	g_free (cmd);
 }
 
+void
+totem_action_show_properties (Totem *totem)
+{
+	totem_sidebar_set_current_page (totem, "properties");
+}
+
 void
 totem_action_play (Totem *totem)
 {
@@ -347,7 +362,7 @@ totem_action_play (Totem *totem)
 	retval = bacon_video_widget_play (totem->bvw,  &err);
 	play_pause_set_label (totem, retval ? STATE_PLAYING : STATE_STOPPED);
 	if (totem_is_fullscreen (totem) != FALSE)
-		totem_scrsaver_disable (totem->scr);
+		totem_scrsaver_set_state (totem->scr, !retval);
 
 	if (retval == FALSE)
 	{
@@ -401,6 +416,40 @@ totem_action_set_mrl_and_play (Totem *totem, const char *mrl)
 		totem_action_play (totem);
 }
 
+static gboolean
+totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
+{
+	GSList *filenames;
+	gboolean playlist_modified;
+
+	filenames = totem_add_files (GTK_WINDOW (totem->win), path);
+
+	if (filenames == NULL)
+		return FALSE;
+
+	playlist_modified = totem_action_open_files_list (totem,
+			filenames);
+
+	if (playlist_modified == FALSE) {
+		g_slist_foreach (filenames, (GFunc) g_free, NULL);
+		g_slist_free (filenames);
+		return FALSE;
+	}
+
+	g_slist_foreach (filenames, (GFunc) g_free, NULL);
+	g_slist_free (filenames);
+
+	if (play != FALSE) {
+		char *mrl;
+
+		mrl = totem_playlist_get_current_mrl (totem->playlist);
+		totem_action_set_mrl_and_play (totem, mrl);
+		g_free (mrl);
+	}
+
+	return TRUE;
+}
+
 static gboolean
 totem_action_load_media (Totem *totem, MediaType type)
 {
@@ -456,12 +505,7 @@ totem_action_load_media_device (Totem *totem, const char *device)
 		case MEDIA_TYPE_DATA:
 			/* Set default location to the mountpoint of
 			 * this device */
-			{
-				char *s;
-				s = totem_action_open_dialog (totem, url, FALSE);
-				retval = (s != NULL);
-				g_free (s);
-			}
+			retval = totem_action_open_dialog (totem, url, FALSE);
 			break;
 		case MEDIA_TYPE_DVD:
 		case MEDIA_TYPE_VCD:
@@ -590,6 +634,7 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 
 		totem->controls_visibility = TOTEM_CONTROLS_FULLSCREEN;
 		show_controls (totem, FALSE);
+		totem_action_set_sensitivity ("fullscreen", FALSE);
 	} else {
 		GtkAction *action;
 
@@ -600,16 +645,15 @@ window_state_event_cb (GtkWidget *window, GdkEventWindowState *event,
 		totem_scrsaver_enable (totem->scr);
 
 		action = gtk_action_group_get_action (totem->main_action_group,
-			"show-controls");
+				"show-controls");
 
 		if (gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)))
-		{
 			totem->controls_visibility = TOTEM_CONTROLS_VISIBLE;
-			show_controls (totem, TRUE);
-		} else {
+		else
 			totem->controls_visibility = TOTEM_CONTROLS_HIDDEN;
-			show_controls (totem, TRUE);
-		}
+
+		show_controls (totem, TRUE);
+		totem_action_set_sensitivity ("fullscreen", TRUE);
 	}
 
 	return FALSE;
@@ -637,44 +681,144 @@ totem_action_fullscreen (Totem *totem, gboolean state)
 void
 totem_action_open (Totem *totem)
 {
-	static char *path = NULL;
-	char *new_path;
+	totem_action_open_dialog (totem, NULL, TRUE);
+}
 
-	new_path = totem_action_open_dialog (totem, path, TRUE);
-	if (new_path != NULL) {
-		g_free (path);
-		path = new_path;
+static gint
+totem_compare_recent_stream_items (GtkRecentInfo *a, GtkRecentInfo *b)
+{
+	gboolean has_totem_a, has_totem_b;
+
+	has_totem_a = gtk_recent_info_has_group (a, "TotemStreams");
+	has_totem_b = gtk_recent_info_has_group (b, "TotemStreams");
+
+	if (has_totem_a && has_totem_b) {
+		time_t time_a, time_b;
+
+		time_a = gtk_recent_info_get_modified (a);
+		time_b = gtk_recent_info_get_modified (b);
+
+		return (time_b - time_a);
+	} else if (has_totem_a) {
+		return -1;
+	} else if (has_totem_b) {
+		return 1;
 	}
+
+	return 0;
+}
+
+static char *
+totem_open_location_set_from_clipboard (Totem *totem)
+{
+	GtkClipboard *clipboard;
+	gchar *clipboard_content;
+
+	/* Initialize the clipboard and get its content */
+	clipboard = gtk_clipboard_get_for_display (gtk_widget_get_display (totem->win), GDK_SELECTION_CLIPBOARD);
+	clipboard_content = gtk_clipboard_wait_for_text (clipboard);
+
+	/* Check clipboard for "://". If it exists, return it */
+	if (clipboard_content != NULL && strcmp (clipboard_content, "") != 0)
+	{
+		if (g_strrstr (clipboard_content, "://") != NULL)
+			return clipboard_content;
+	}
+
+	g_free (clipboard_content);
+	return NULL;
+}
+
+static gboolean
+totem_open_location_match (GtkEntryCompletion *completion, const gchar *key, GtkTreeIter *iter, gpointer user_data)
+{
+	/* Substring-match key against uri */
+	char *uri, *match;
+
+	g_return_val_if_fail (key != NULL, FALSE);
+	gtk_tree_model_get (user_data, iter, 0, &uri, -1);
+	g_return_val_if_fail (uri != NULL, FALSE);
+	match = strstr (uri, key);
+	g_free (uri);
+
+	return (match != NULL);
 }
 
-//FIXME move the URI to a separate widget
 void
 totem_action_open_location (Totem *totem)
 {
 	GladeXML *glade;
-	char *mrl;
-	GtkWidget *dialog, *entry, *gentry;
+	char *mrl, *clipboard_location;
+	GtkWidget *dialog, *entry;
 	int response;
 	const char *filenames[2];
+	GtkEntryCompletion *completion;
+	GtkTreeModel *model;
+	GList *recent_items;
 
 	glade = totem_interface_load ("uri.glade", _("Open Location..."),
 			FALSE, GTK_WINDOW (totem->win));
 	if (glade == NULL)
 		return;
 
+	/* Get item from clipboard to fill entry in Open Location... dialog */
+	clipboard_location = totem_open_location_set_from_clipboard (totem);
+	if (clipboard_location != NULL && strcmp (clipboard_location, "") != 0)
+		gtk_entry_set_text (GTK_ENTRY (glade_xml_get_widget (glade, "uri")), clipboard_location);
+	g_free (clipboard_location);
+
 	dialog = glade_xml_get_widget (glade, "open_uri_dialog");
 	entry = glade_xml_get_widget (glade, "uri");
-	gentry = glade_xml_get_widget (glade, "uri_list");
+
+	completion = gtk_entry_completion_new();
+	model = GTK_TREE_MODEL (gtk_list_store_new (1, G_TYPE_STRING));
+	gtk_entry_set_completion (GTK_ENTRY (entry), completion);
+
+	/* Add items in Totem's GtkRecentManager to the uri_list GtkEntry's GtkEntryCompletion */
+	recent_items = gtk_recent_manager_get_items (totem->recent_manager);
+
+	if (recent_items != NULL)
+	{
+		GList *p;
+		GtkTreeIter iter;
+
+		recent_items = g_list_sort (recent_items, (GCompareFunc) totem_compare_recent_stream_items);
+
+		for (p = recent_items; p != NULL; p = p->next)
+		{
+			if (!gtk_recent_info_has_group ((GtkRecentInfo *) p->data, "TotemStreams"))
+				continue;
+
+			gtk_list_store_append (GTK_LIST_STORE (model), &iter);
+			gtk_list_store_set (GTK_LIST_STORE (model), &iter, 0, gtk_recent_info_get_uri ((GtkRecentInfo *) p->data), -1);
+			gtk_recent_info_unref ((GtkRecentInfo *) p->data);
+		}
+	}
+
+	g_list_free (recent_items);
+
+	gtk_entry_completion_set_model (completion, model);
+	gtk_entry_completion_set_text_column (completion, 0);
+	gtk_entry_completion_set_match_func (completion, (GtkEntryCompletionMatchFunc) totem_open_location_match, model, NULL);
 
 	response = gtk_dialog_run (GTK_DIALOG (dialog));
 
 	if (response == GTK_RESPONSE_OK)
 	{
-		const char *uri;
+		char *uri;
+
+		uri = g_strdup (gtk_entry_get_text (GTK_ENTRY (entry)));
 
-		uri = gtk_entry_get_text (GTK_ENTRY (entry));
 		if (uri != NULL && strcmp (uri, "") != 0)
 		{
+			if (g_strrstr (uri, "://") == NULL)
+			{
+				char *tmp;
+				tmp = g_strconcat ("http://", uri, NULL);
+				g_free (uri);
+				uri = tmp;
+			}
+
 			filenames[0] = uri;
 			filenames[1] = NULL;
 			totem_action_open_files (totem, (char **) filenames);
@@ -682,11 +826,8 @@ totem_action_open_location (Totem *totem)
 			mrl = totem_playlist_get_current_mrl (totem->playlist);
 			totem_action_set_mrl_and_play (totem, mrl);
 			g_free (mrl);
-#ifndef HAVE_GTK_ONLY
-			gnome_entry_append_history (GNOME_ENTRY (gentry),
-					TRUE, uri);
-#endif /* !HAVE_GTK_ONLY */
 		}
+		g_free (uri);
 	}
 
 	gtk_widget_destroy (dialog);
@@ -728,7 +869,7 @@ totem_action_take_screenshot (Totem *totem)
 
 	gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_destroy (dialog);
-	gdk_pixbuf_unref (pixbuf);
+	g_object_unref (pixbuf);
 }
 
 static char *
@@ -870,6 +1011,9 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 		/* Take a screenshot */
 		totem_action_set_sensitivity ("take-screenshot", FALSE);
 
+		/* Clear the playlist */
+		totem_action_set_sensitivity ("clear-playlist", FALSE);
+
 		/* Set the logo */
 		bacon_video_widget_set_logo_mode (totem->bvw, TRUE);
 		gtk_widget_set_sensitive (GTK_WIDGET (totem->properties),
@@ -906,6 +1050,10 @@ totem_action_set_mrl_with_warning (Totem *totem, const char *mrl,
 
 		/* Take a screenshot */
 		totem_action_set_sensitivity ("take-screenshot", retval);
+
+		/* Clear the playlist */
+		totem_action_set_sensitivity ("clear-playlist", retval);
+
 		gtk_widget_set_sensitive
 			(GTK_WIDGET (totem->properties), retval);
 
@@ -1042,23 +1190,33 @@ static void
 totem_action_zoom (Totem *totem, int zoom)
 {
 	GtkAction *action;
+	gboolean zoom_reset, zoom_in, zoom_out;
 
-	if (zoom < ZOOM_LOWER || zoom > ZOOM_UPPER)
+	if (zoom == ZOOM_ENABLE)
+		zoom = bacon_video_widget_get_zoom (totem->bvw);
+
+	if (zoom == ZOOM_DISABLE) {
+		zoom_reset = zoom_in = zoom_out = FALSE;
+	} else if (zoom < ZOOM_LOWER || zoom > ZOOM_UPPER) {
 		return;
-
-	bacon_video_widget_set_zoom (totem->bvw, zoom);
+	} else {
+		bacon_video_widget_set_zoom (totem->bvw, zoom);
+		zoom_reset = (zoom != ZOOM_RESET);
+		zoom_out = zoom != ZOOM_LOWER;
+		zoom_in = zoom != ZOOM_UPPER;
+	}
 
 	action = gtk_action_group_get_action (totem->zoom_action_group,
 			"zoom-in");
-	gtk_action_set_sensitive (action, zoom != ZOOM_UPPER);
+	gtk_action_set_sensitive (action, zoom_in);
 
 	action = gtk_action_group_get_action (totem->zoom_action_group,
 			"zoom-out");
-	gtk_action_set_sensitive (action, zoom != ZOOM_LOWER);
+	gtk_action_set_sensitive (action, zoom_out);
 
 	action = gtk_action_group_get_action (totem->zoom_action_group,
 			"zoom-reset");
-	gtk_action_set_sensitive (action, zoom != ZOOM_RESET);
+	gtk_action_set_sensitive (action, zoom_reset);
 }
 
 void
@@ -1399,12 +1557,6 @@ on_error_event (BaconVideoWidget *bvw, char *message,
 	}
 }
 
-static void
-on_speed_warning_event (BaconVideoWidget *bvw, Totem *totem)
-{
-	g_message ("TBD: Implement speed warning");
-}
-
 static void
 on_buffering_event (BaconVideoWidget *bvw, int percentage, Totem *totem)
 {
@@ -1513,15 +1665,16 @@ update_volume_sliders (Totem *totem)
 	totem->prev_volume = volume;
 }
 
-static int
-gui_update_cb (Totem *totem)
+static void
+property_notify_cb (BaconVideoWidget *bvw, GParamSpec *spec, Totem *totem)
 {
-	if (totem->bvw == NULL)
-		return TRUE;
-
-	update_volume_sliders (totem);
-
-	return TRUE;
+	if (strcmp ("volume", spec->name) == 0) {
+		update_volume_sliders (totem);
+	} else if (strcmp ("logo-mode", spec->name) == 0) {
+		gboolean enabled;
+		enabled = bacon_video_widget_get_logo_mode (totem->bvw);
+		totem_action_zoom (totem, enabled ? ZOOM_DISABLE : ZOOM_ENABLE);
+	}
 }
 
 static gboolean
@@ -1531,16 +1684,6 @@ seek_slider_pressed_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 	totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
 	totem_time_label_set_seeking (TOTEM_TIME_LABEL (totem->tcw_time_label), TRUE);
 
-	if (bacon_video_widget_can_direct_seek (totem->bvw) != FALSE)
-	{
-		if (!totem->seek_in_progress) {
-			totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
-			bacon_video_widget_pause (totem->bvw);
-			/* We need to remember that we are already paused and seeking */
-			totem->seek_in_progress = TRUE;
-		}
-	}
-
 	return FALSE;
 }
 
@@ -1582,12 +1725,8 @@ seek_slider_released_cb (GtkWidget *widget, GdkEventButton *event, Totem *totem)
 	val = gtk_adjustment_get_value (adj);
 	gtk_adjustment_set_value (other_adj, val);
 
-	if (bacon_video_widget_can_direct_seek (totem->bvw) != FALSE)
+	if (bacon_video_widget_can_direct_seek (totem->bvw) == FALSE)
 	{
-		if (totem->was_playing)
-			bacon_video_widget_play (totem->bvw, NULL);
-		totem->seek_in_progress = FALSE;
-	} else {
 		totem_action_seek (totem, val / 65535.0);
 	}
 
@@ -1646,10 +1785,14 @@ static gboolean
 totem_action_open_files_list (Totem *totem, GSList *list)
 {
 	GSList *l;
-	gboolean cleared = FALSE;
+	gboolean changed;
+	gboolean cleared;
+
+	changed = FALSE;
+	cleared = FALSE;
 
 	if (list == NULL)
-		return cleared;
+		return changed;
 
 	for (l = list ; l != NULL; l = l->next)
 	{
@@ -1682,18 +1825,21 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 				g_signal_handlers_disconnect_by_func
 					(G_OBJECT (totem->playlist),
 					 playlist_changed_cb, totem);
-				totem_playlist_clear (totem->playlist);
+				changed = totem_playlist_clear (totem->playlist);
 				bacon_video_widget_close (totem->bvw);
 				cleared = TRUE;
 			}
 
 			if (totem_is_block_device (filename) != FALSE) {
 				totem_action_load_media_device (totem, data);
-			} else if (strstr (filename, "cdda:/") != NULL) {
+				changed = TRUE;
+			} else if (g_str_has_prefix (filename, "cdda:/") != FALSE) {
 				totem_playlist_add_mrl (totem->playlist, data, NULL);
+				changed = TRUE;
 			} else if (totem_playlist_add_mrl (totem->playlist,
 						filename, NULL) != FALSE) {
 				totem_action_add_recent (totem, filename);
+				changed = TRUE;
 			}
 		}
 
@@ -1708,43 +1854,7 @@ totem_action_open_files_list (Totem *totem, GSList *list)
 				totem);
 	}
 
-	return cleared;
-}
-
-char *
-totem_action_open_dialog (Totem *totem, const char *path, gboolean play)
-{
-	GSList *filenames;
-	char *new_path;
-	gboolean playlist_modified;
-
-	filenames = totem_add_files (GTK_WINDOW (totem->win),
-			path, &new_path);
-
-	if (filenames == NULL)
-		return NULL;
-
-	playlist_modified = totem_action_open_files_list (totem,
-			filenames);
-
-	if (playlist_modified == FALSE) {
-		g_slist_foreach (filenames, (GFunc) g_free, NULL);
-		g_slist_free (filenames);
-		return NULL;
-	}
-
-	g_slist_foreach (filenames, (GFunc) g_free, NULL);
-	g_slist_free (filenames);
-
-	if (play != FALSE) {
-		char *mrl;
-
-		mrl = totem_playlist_get_current_mrl (totem->playlist);
-		totem_action_set_mrl_and_play (totem, mrl);
-		g_free (mrl);
-	}
-
-	return new_path;
+	return changed;
 }
 
 static void
@@ -1784,6 +1894,7 @@ void
 totem_action_skip_to (Totem *totem)
 {
 	char *filename;
+	TotemSkipto **skipto;
 
 	if (totem->seekable == FALSE)
 		return;
@@ -1802,8 +1913,11 @@ totem_action_skip_to (Totem *totem)
 			G_CALLBACK (gtk_widget_destroy), NULL);
 	g_signal_connect (G_OBJECT (totem->skipto), "response",
 			G_CALLBACK (commit_hide_skip_to), totem);
+
+	skipto = &totem->skipto;
 	g_object_add_weak_pointer (G_OBJECT (totem->skipto),
-			(gpointer *)&totem->skipto);
+				   (gpointer *) skipto);
+
 	gtk_window_set_transient_for (GTK_WINDOW (totem->skipto),
 			GTK_WINDOW (totem->win));
 	gtk_widget_show (GTK_WIDGET (totem->skipto));
@@ -1846,17 +1960,40 @@ show_controls (Totem *totem, gboolean was_fullscreen)
 		gtk_widget_show (menubar);
 		gtk_widget_show (controlbar);
 		gtk_widget_show (statusbar);
-		if (totem_sidebar_is_visible (totem) != FALSE)
+		if (totem_sidebar_is_visible (totem) != FALSE) {
+			/* This is uglier then you might expect because of the
+			   resize handle between the video and sidebar. There
+			   is no convenience method to get the handle's width.
+			   */
+			GValue value = { 0, };
+			GtkWidget *pane;
+			int handle_size;
+
+			g_value_init (&value, G_TYPE_INT);
+			pane = glade_xml_get_widget (totem->xml,
+					"tmw_main_pane");
+			gtk_widget_style_get_property (pane, "handle-size",
+					&value);
+			handle_size = g_value_get_int (&value);
+			
 			gtk_widget_show (totem->sidebar);
-		else
+			width += totem->sidebar->allocation.width
+				+ handle_size;
+		} else {
 			gtk_widget_hide (totem->sidebar);
+		}
 
 		gtk_container_set_border_width (GTK_CONTAINER (bvw_vbox),
 				BVW_VBOX_BORDER_WIDTH);
 
 		if (was_fullscreen == FALSE)
 		{
-			totem_widget_set_preferred_size (widget,
+			height += menubar->allocation.height
+				+ controlbar->allocation.height
+				+ statusbar->allocation.height
+				+ 2 * BVW_VBOX_BORDER_WIDTH;
+			width += 2 * BVW_VBOX_BORDER_WIDTH;
+			gtk_window_resize (GTK_WINDOW(totem->win),
 					width, height);
 		}
 	} else {
@@ -1980,14 +2117,14 @@ totem_action_remote (Totem *totem, TotemRemoteCommand cmd, const char *url)
 	case TOTEM_REMOTE_COMMAND_TOGGLE_CONTROLS:
 		if (totem->controls_visibility != TOTEM_CONTROLS_FULLSCREEN)
 		{
-			GtkCheckMenuItem *item;
-			gboolean value;
+			GtkToggleAction *action;
+			gboolean state;
 
-			item = GTK_CHECK_MENU_ITEM (glade_xml_get_widget
-					(totem->xml,
-					 "tmw_show_controls_menu_item"));
-			value = gtk_check_menu_item_get_active (item);
-			gtk_check_menu_item_set_active (item, !value);
+			action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
+					(totem->main_action_group,
+					 "show-controls"));
+			state = gtk_toggle_action_get_active (action);
+			gtk_toggle_action_set_active (action, !state);
 		}
 		break;
 	case TOTEM_REMOTE_COMMAND_SHOW_PLAYING:
@@ -2210,6 +2347,18 @@ theme_changed_cb (GtkIconTheme *icon_theme, Totem *totem)
 	move_popups (totem);
 }
 
+static void
+window_realize_cb (GtkWidget *widget, Totem *totem)
+{
+	GdkScreen *screen;
+
+	screen = gtk_widget_get_screen (widget);
+	g_signal_connect (G_OBJECT (screen), "size-changed",
+			G_CALLBACK (size_changed_cb), totem);
+	g_signal_connect (G_OBJECT (gtk_icon_theme_get_for_screen (screen)),
+			"changed", G_CALLBACK (theme_changed_cb), totem);
+}
+
 static void
 popup_timeout_add(Totem* totem)
 {
@@ -2289,8 +2438,13 @@ static gboolean
 on_video_button_press_event (BaconVideoWidget *bvw, GdkEventButton *event,
 		Totem *totem)
 {
-	if (event->type == GDK_BUTTON_PRESS && event->button == 3)
-	{
+	if (event->type == GDK_2BUTTON_PRESS && event->button == 1) {
+		totem_action_fullscreen_toggle(totem);
+		return TRUE;
+	} else if (event->type == GDK_BUTTON_PRESS && event->button == 2) {
+		totem_action_play_pause(totem);
+		return TRUE;
+	} else if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
 		totem_action_menu_popup (totem, event->button);
 		return TRUE;
 	}
@@ -2330,29 +2484,57 @@ totem_action_handle_key_release (Totem *totem, GdkEventKey *event)
 {
 	gboolean retval = TRUE;
 
-	/* Alphabetical */
 	switch (event->keyval) {
 	case GDK_Left:
 	case GDK_Right:
-		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), FALSE);
-		if (bacon_video_widget_can_direct_seek (totem->bvw)) {
-			if (totem->was_playing) {
-				bacon_video_widget_play (totem->bvw, NULL);
-			}
-			totem->seek_in_progress = FALSE;
-		}
+		totem_statusbar_set_seeking
+			(TOTEM_STATUSBAR (totem->statusbar), FALSE);
 		break;
 	}
 
 	return retval;
 }
 
+static void
+totem_action_handle_seek (Totem *totem, GdkEventKey *event, gboolean is_forward)
+{
+	if (is_forward != FALSE) {
+		if (event->state & GDK_SHIFT_MASK) {
+			totem_action_seek_relative (totem,
+					SEEK_FORWARD_SHORT_OFFSET);
+		} else if (event->state & GDK_CONTROL_MASK) {
+			totem_action_seek_relative (totem,
+					SEEK_FORWARD_LONG_OFFSET);
+		} else {
+			totem_action_seek_relative (totem,
+					SEEK_FORWARD_OFFSET);
+		}
+	} else {
+		if (event->state & GDK_SHIFT_MASK) {
+			totem_action_seek_relative (totem,
+					SEEK_BACKWARD_SHORT_OFFSET);
+		} else if (event->state & GDK_CONTROL_MASK) {
+			totem_action_seek_relative (totem,
+					SEEK_BACKWARD_LONG_OFFSET);
+		} else {
+			totem_action_seek_relative (totem,
+					SEEK_BACKWARD_OFFSET);
+		}
+	}
+}
+
 static gboolean
 totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 {
-	gboolean retval = TRUE;
+	gboolean retval = TRUE, playlist_focused = FALSE;
+	GtkWidget *focused;
+
+	focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
+	if (focused != NULL && gtk_widget_is_ancestor
+			(focused, GTK_WIDGET (totem->playlist)) != FALSE) {
+		playlist_focused = TRUE;
+	}
 
-	/* Alphabetical */
 	switch (event->keyval) {
 	case GDK_A:
 	case GDK_a:
@@ -2399,14 +2581,14 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 	case GDK_i:
 	case GDK_I:
 		{
-			GtkCheckMenuItem *item;
-			gboolean value;
+			GtkToggleAction *action;
+			gboolean state;
 
-			item = GTK_CHECK_MENU_ITEM (glade_xml_get_widget
-					(totem->xml,
-					 "tmw_deinterlace_menu_item"));
-			value = gtk_check_menu_item_get_active (item);
-			gtk_check_menu_item_set_active (item, !value);
+			action = GTK_TOGGLE_ACTION (gtk_action_group_get_action
+					(totem->main_action_group,
+					 "deinterlace"));
+			state = gtk_toggle_action_get_active (action);
+			gtk_toggle_action_set_active (action, !state);
 		}
 		break;
 	case GDK_M:
@@ -2431,7 +2613,10 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 #endif /* HAVE_XFREE */
 	case GDK_p:
 	case GDK_P:
-		totem_action_play_pause (totem);
+		if (event->state & GDK_CONTROL_MASK)
+			totem_action_show_properties (totem);
+		else 
+			totem_action_play_pause (totem);
 		break;
 	case GDK_q:
 	case GDK_Q:
@@ -2460,46 +2645,26 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 			totem_action_fullscreen (totem, FALSE);
 		break;
 	case GDK_Left:
-		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
-		if (bacon_video_widget_can_direct_seek (totem->bvw)) {
-			if (!totem->seek_in_progress) {
-				totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
-				bacon_video_widget_pause (totem->bvw);
-				/* We need to remember that we are already paused and seeking */
-				totem->seek_in_progress = TRUE;
-			}
-		}
-		if (event->state & GDK_SHIFT_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_BACKWARD_SHORT_OFFSET);
-		} else if (event->state & GDK_CONTROL_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_BACKWARD_LONG_OFFSET);
-		} else {
-			totem_action_seek_relative (totem,
-					SEEK_BACKWARD_OFFSET);
-		}
+		if (playlist_focused != FALSE)
+			return FALSE;
+
+		totem_statusbar_set_seeking
+			(TOTEM_STATUSBAR (totem->statusbar), TRUE);
+		if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL)
+			totem_action_handle_seek (totem, event, TRUE);
+		else
+			totem_action_handle_seek (totem, event, FALSE);
 		break;
 	case GDK_Right:
-		totem_statusbar_set_seeking (TOTEM_STATUSBAR (totem->statusbar), TRUE);
-		if (bacon_video_widget_can_direct_seek (totem->bvw)) {
-			if (!totem->seek_in_progress) {
-				totem->was_playing = bacon_video_widget_is_playing (totem->bvw);
-				bacon_video_widget_pause (totem->bvw);
-				/* We need to remember that we are already paused and seeking */
-				totem->seek_in_progress = TRUE;
-			}
-		}
-		if (event->state & GDK_SHIFT_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_FORWARD_SHORT_OFFSET);
-		} else if (event->state & GDK_CONTROL_MASK) {
-			totem_action_seek_relative (totem,
-					SEEK_FORWARD_LONG_OFFSET);
-		} else {
-			totem_action_seek_relative (totem,
-					SEEK_FORWARD_OFFSET);
-		}
+		if (playlist_focused != FALSE)
+			return FALSE;
+
+		totem_statusbar_set_seeking
+			(TOTEM_STATUSBAR (totem->statusbar), TRUE);
+		if (gtk_widget_get_direction (totem->win) == GTK_TEXT_DIR_RTL)
+			totem_action_handle_seek (totem, event, FALSE);
+		else
+			totem_action_handle_seek (totem, event, TRUE);
 		break;
 	case GDK_space:
 		if (totem_is_fullscreen (totem) != FALSE || gtk_widget_is_focus (GTK_WIDGET (totem->bvw)) != FALSE)
@@ -2508,17 +2673,20 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 			retval = FALSE;
 		break;
 	case GDK_Up:
+		if (playlist_focused != FALSE)
+			return FALSE;
 		totem_action_volume_relative (totem, VOLUME_UP_OFFSET);
 		break;
 	case GDK_Down:
+		if (playlist_focused != FALSE)
+			return FALSE;
 		totem_action_volume_relative (totem, VOLUME_DOWN_OFFSET);
 		break;
 	case GDK_0:
-		if (event->state & GDK_CONTROL_MASK) {
+		if (event->state & GDK_CONTROL_MASK)
 			totem_action_zoom_reset (totem);
-		} else {
+		else
 			totem_action_set_scale_ratio (totem, 0.5);
-		}
 		break;
 	case GDK_onehalf:
 		totem_action_set_scale_ratio (totem, 0.5);
@@ -2530,18 +2698,15 @@ totem_action_handle_key_press (Totem *totem, GdkEventKey *event)
 		totem_action_set_scale_ratio (totem, 2);
 		break;
 	case GDK_Menu:
+		if (playlist_focused != FALSE)
+			return FALSE;
+		totem_action_menu_popup (totem, 0);
+		break;
 	case GDK_F10:
-		/* Chain up if the playlist has the focuse */
-		{
-			GtkWidget *focused;
-
-			if (event->keyval != GDK_Menu && !(event->state & GDK_SHIFT_MASK))
-				return FALSE;
-
-			focused = gtk_window_get_focus (GTK_WINDOW (totem->win));
-			if (focused != NULL && gtk_widget_is_ancestor (focused, GTK_WIDGET (totem->playlist)) != FALSE)
-				return FALSE;
-		}
+		if (playlist_focused != FALSE)
+			return FALSE;
+		if (!(event->state & GDK_SHIFT_MASK))
+			return FALSE;
 
 		totem_action_menu_popup (totem, 0);
 		break;
@@ -2721,7 +2886,6 @@ update_media_menu_items (Totem *totem)
 	totem_action_set_sensitivity ("next-angle", playing);
 
 	playing = totem_is_media (totem->mrl);
-
 	totem_action_set_sensitivity ("eject", playing);
 }
 
@@ -2747,20 +2911,37 @@ update_buttons (Totem *totem)
 	totem_action_set_sensitivity ("next-chapter", has_item);
 }
 
+static void
+main_pane_size_allocated (GtkWidget *main_pane, GtkAllocation *allocation, Totem *totem)
+{
+	gulong handler_id;
+
+	if (!totem->maximised || GTK_WIDGET_MAPPED (totem->win)) {
+		handler_id = g_signal_handler_find (main_pane, 
+				G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA,
+				0, 0, NULL,
+				main_pane_size_allocated, totem);
+		g_signal_handler_disconnect (main_pane, handler_id);
+
+		gtk_paned_set_position (GTK_PANED (main_pane), allocation->width - totem->sidebar_w);
+	}
+}
+
 static void
 totem_setup_window (Totem *totem)
 {
 	GKeyFile *keyfile;
-	int w, h, sidebar_w;
+	int w, h;
 	gboolean show_sidebar;
 	char *filename, *page_id;
 	GError *err = NULL;
+	GtkWidget *main_pane;
 
 	filename = g_build_filename (g_get_home_dir (), ".gnome2", "totem", NULL);
 	keyfile = g_key_file_new ();
 	if (g_key_file_load_from_file (keyfile, filename,
 			G_KEY_FILE_NONE, NULL) == FALSE) {
-		sidebar_w = w = h = 0;
+		totem->sidebar_w = w = h = 0;
 		show_sidebar = TRUE;
 		page_id = NULL;
 		g_free (filename);
@@ -2804,11 +2985,11 @@ totem_setup_window (Totem *totem)
 			err = NULL;
 		}
 
-		sidebar_w = g_key_file_get_integer (keyfile, "State",
+		totem->sidebar_w = g_key_file_get_integer (keyfile, "State",
 				"sidebar_w", &err);
 		if (err != NULL) {
 			g_error_free (err);
-			sidebar_w = 0;
+			totem->sidebar_w = 0;
 		}
 		g_key_file_free (keyfile);
 	}
@@ -2821,17 +3002,11 @@ totem_setup_window (Totem *totem)
 	} else if (totem->maximised != FALSE) {
 		gtk_window_maximize (GTK_WINDOW (totem->win));
 	}
+	
+	main_pane = glade_xml_get_widget (totem->xml, "tmw_main_pane");
+	g_signal_connect (G_OBJECT (main_pane), "size-allocate", G_CALLBACK (main_pane_size_allocated), totem);
 
-	if (sidebar_w > 0 && totem->maximised == FALSE) {
-		GtkWidget *item;
-
-		totem->sidebar_w = sidebar_w;
-		totem_sidebar_setup (totem, show_sidebar, page_id);
-		item = glade_xml_get_widget (totem->xml, "tmw_main_pane");
-		gtk_paned_set_position (GTK_PANED (item), w - sidebar_w);
-	} else {
-		totem_sidebar_setup (totem, show_sidebar, page_id);
-	}
+	totem_sidebar_setup (totem, show_sidebar, page_id);
 }
 
 static void
@@ -2851,24 +3026,27 @@ totem_callback_connect (Totem *totem)
 	gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (action),
 		totem_playlist_get_shuffle (totem->playlist));
 
-#ifdef HAVE_GTK_ONLY
-	action = gtk_action_group_get_action (totem->main_action_group, "contents");
-	gtk_action_set_visible (action, FALSE)
-#endif /* HAVE_GTK_ONLY */
-
 	/* Controls */
 	box = glade_xml_get_widget (totem->xml, "tmw_buttons_hbox");
 
-	action = gtk_action_group_get_action (totem->main_action_group, "previous-chapter");
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"previous-chapter");
 	item = gtk_action_create_tool_item (action);
+	atk_object_set_name (gtk_widget_get_accessible (item),
+			_("Previous Chapter/Movie"));
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
 
 	action = gtk_action_group_get_action (totem->main_action_group, "play");
 	item = gtk_action_create_tool_item (action);
+	atk_object_set_name (gtk_widget_get_accessible (item),
+			_("Play / Pause"));
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
 
-	action = gtk_action_group_get_action (totem->main_action_group, "next-chapter");
+	action = gtk_action_group_get_action (totem->main_action_group,
+			"next-chapter");
 	item = gtk_action_create_tool_item (action);
+	atk_object_set_name (gtk_widget_get_accessible (item),
+			_("Next Chapter/Movie"));
 	gtk_box_pack_start (GTK_BOX (box), item, FALSE, FALSE, 0);
 
 	/* Sidebar button (Drag'n'Drop) */
@@ -2896,10 +3074,8 @@ totem_callback_connect (Totem *totem)
 			G_CALLBACK (window_state_event_cb), totem);
 
 	/* Screen size and Theme changes */
-	g_signal_connect (G_OBJECT (gdk_screen_get_default ()),
-			"size-changed", G_CALLBACK (size_changed_cb), totem);
-	g_signal_connect (G_OBJECT (gtk_icon_theme_get_default ()),
-			"changed", G_CALLBACK (theme_changed_cb), totem);
+	g_signal_connect (totem->win, "realize",
+			G_CALLBACK (window_realize_cb), totem);
 
 	/* Motion notify for the Popups */
 	item = glade_xml_get_widget (totem->xml,
@@ -2996,16 +3172,10 @@ totem_callback_connect (Totem *totem)
 	g_signal_connect (G_OBJECT (totem->volume), "value-changed",
 			G_CALLBACK (vol_cb), totem);
 
-	/* Named icon support */
-	totem_set_default_icons (totem);
-
 	/* Set sensitivity of the toolbar buttons */
 	totem_action_set_sensitivity ("play", FALSE);
 	totem_action_set_sensitivity ("next-chapter", FALSE);
 	totem_action_set_sensitivity ("previous-chapter", FALSE);
-
-	/* Update the UI */
-	g_timeout_add (600, (GSourceFunc) gui_update_cb, totem);
 }
 
 static void
@@ -3043,7 +3213,8 @@ static void
 video_widget_create (Totem *totem) 
 {
 	GError *err = NULL;
-	GtkWidget *container, *container2;
+	GtkWidget *container;
+	BaconVideoWidget **bvw;
 
 	totem->scr = totem_scrsaver_new ();
 
@@ -3069,7 +3240,7 @@ video_widget_create (Totem *totem)
 			"motion-notify-event",
 			G_CALLBACK (on_video_motion_notify_event),
 			totem);
-	g_signal_connect (G_OBJECT (totem->bvw),
+	g_signal_connect_after (G_OBJECT (totem->bvw),
 			"button-press-event",
 			G_CALLBACK (on_video_button_press_event),
 			totem);
@@ -3105,10 +3276,8 @@ video_widget_create (Totem *totem)
 			"error",
 			G_CALLBACK (on_error_event),
 			totem);
-	g_signal_connect (G_OBJECT (totem->bvw),
-			"speed-warning",
-			G_CALLBACK (on_speed_warning_event),
-			totem);
+
+	totem_missing_plugins_setup (totem);
 
 	container = glade_xml_get_widget (totem->xml, "tmw_bvw_vbox");
 	gtk_container_add (GTK_CONTAINER (container),
@@ -3135,17 +3304,17 @@ video_widget_create (Totem *totem)
 			source_table, G_N_ELEMENTS (source_table),
 			GDK_ACTION_LINK);
 
+	bvw = &(totem->bvw);
 	g_object_add_weak_pointer (G_OBJECT (totem->bvw),
-			(void**)&(totem->bvw));
+				   (gpointer *) bvw);
 
 	gtk_widget_show (GTK_WIDGET (totem->bvw));
 
-	container2 = glade_xml_get_widget (totem->xml, "tmw_main_pane");
-	gtk_paned_set_position (GTK_PANED (container2), totem->window_w - totem->sidebar_w);
-
 	bacon_video_widget_set_volume (totem->bvw,
 			gconf_client_get_int (totem->gc,
 				GCONF_PREFIX"/volume", NULL));
+	g_signal_connect (G_OBJECT (totem->bvw), "notify",
+			G_CALLBACK (property_notify_cb), totem);
 	update_volume_sliders (totem);
 }
 
@@ -3190,28 +3359,44 @@ main (int argc, char **argv)
 {
 	Totem *totem;
 	GConfClient *gc;
+#ifndef HAVE_GTK_ONLY
+	GnomeProgram *program;
+#endif
+	GOptionContext *context;
+	GOptionGroup *baconoptiongroup;
 
 	bindtextdomain (GETTEXT_PACKAGE, GNOMELOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+#ifdef GDK_WINDOWING_X11
 	if (XInitThreads () == 0)
 	{
 		gtk_init (&argc, &argv);
 		g_set_application_name (_("Totem Movie Player"));
 		totem_action_error_and_exit (_("Could not initialize the thread-safe libraries."), _("Verify your system installation. Totem will now exit."), NULL);
 	}
+#endif
 
 	g_thread_init (NULL);
 
+	/* Handle command line arguments */
+	context = g_option_context_new (_("- Play movies and songs"));
+	baconoptiongroup = bacon_video_widget_get_option_group();
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, baconoptiongroup);
+
 #ifdef HAVE_GTK_ONLY
-	gtk_init (&argc, &argv);
+	g_option_context_add_group (context, gtk_get_option_group (TRUE));
+	if (g_option_context_parse (context, &argc, argv, &error) == FALSE) {
+		totem_action_error_and_exit (_("Totem could not parse the command-line options"), error->message);
+	}
 #else
-	gnome_program_init ("totem", VERSION,
+	program = gnome_program_init (PACKAGE, VERSION,
 			LIBGNOMEUI_MODULE,
 			argc, argv,
 			GNOME_PARAM_APP_DATADIR, DATADIR,
-			GNOME_PARAM_POPT_TABLE, totem_options_get_options (),
+			GNOME_PARAM_GOPTION_CONTEXT, context,
 			GNOME_PARAM_NONE);
 #endif /* HAVE_GTK_ONLY */
 
@@ -3236,13 +3421,13 @@ main (int argc, char **argv)
 	totem->conn = bacon_message_connection_new (GETTEXT_PACKAGE);
 	if (bacon_message_connection_get_is_server (totem->conn) == FALSE)
 	{
-		totem_options_process_for_server (totem->conn, argc, argv);
+		totem_options_process_for_server (totem->conn, &optionstate);
 		bacon_message_connection_free (totem->conn);
 		g_free (totem);
 		gdk_notify_startup_complete ();
 		exit (0);
 	} else {
-		totem_options_process_early (gc, argc, argv);
+		totem_options_process_early (gc, &optionstate);
 	}
 
 	/* Init totem itself */
@@ -3258,8 +3443,6 @@ main (int argc, char **argv)
 
 	totem->win = glade_xml_get_widget (totem->xml, "totem_main_window");
 	
-	totem_named_icons_init (totem, FALSE);
-
 	/* Menubar */
 	totem_ui_manager_setup (totem);
 
@@ -3321,10 +3504,12 @@ main (int argc, char **argv)
 	totem_setup_preferences (totem);
 
 	/* Command-line handling */
-	if (totem_options_process_late (totem, &argc, &argv) != FALSE)
+	totem_options_process_late (totem, &optionstate);
+
+	if (totem->session_restored != FALSE)
 	{
-		totem_session_restore (totem, argv);
-	} else if (argc >= 1 && totem_action_open_files (totem, argv)) {
+		totem_session_restore (totem, optionstate.filenames);
+	} else if (optionstate.filenames != NULL && totem_action_open_files (totem, optionstate.filenames)) {
 		totem_action_play_pause (totem);
 	} else {
 		totem_action_set_mrl (totem, NULL);
@@ -3346,5 +3531,9 @@ main (int argc, char **argv)
 
 	gtk_main ();
 
+#ifndef HAVE_GTK_ONLY
+	/* Will destroy GOption allocated data automatically */
+	g_object_unref (program);
+#endif	
 	return 0;
 }
